@@ -315,7 +315,8 @@ class Face3D(Base2DIn3D):
     def triangulated_mesh2d(self):
         """A triangulated Mesh2D in the 2D space of the face's plane."""
         if self._mesh2d is None:
-            self._mesh2d = Mesh2D.from_polygon_triangulated(self.polygon2d)
+            self._mesh2d = Mesh2D.from_polygon_triangulated(
+                self.boundary_polygon2d, self.hole_polygon2d)
         return self._mesh2d
 
     @property
@@ -394,7 +395,7 @@ class Face3D(Base2DIn3D):
     def hole_polygon2d(self):
         """A list of Polygon2D for the face holes in the 2D space of the face's plane.
         """
-        if self._hole_polygon2d is None:
+        if self._holes is not None and self._hole_polygon2d is None:
             self._hole_polygon2d = []
             for hole in self.holes:
                 _vert2d = tuple(self._plane.xyz_to_xy(_v) for _v in hole)
@@ -657,7 +658,7 @@ class Face3D(Base2DIn3D):
             return False
         # if it is, convert the point into this face's plane
         vert2d = self.plane.xyz_to_xy(point)
-        return self.polygon2d.is_point_inside(vert2d)
+        return self.polygon2d.is_point_inside(vert2d, Vector2D(1, 0.00001))
 
     def check_planar(self, tolerance, raise_exception=True):
         """Check that all of the face's vertices lie within the face's plane.
@@ -701,16 +702,17 @@ class Face3D(Base2DIn3D):
         """
         if len(self.vertices) == 3:
             return self
-        new_vertices = self._remove_colinear(
-            self._vertices, self.polygon2d, tolerance)
-        _new_face = Face3D(new_vertices, self.plane, enforce_right_hand=False)
-        if not self.has_holes:
+        if not self.has_holes:  # we only need to evaluate one list of vertices
+            new_vertices = self._remove_colinear(
+                self._vertices, self.polygon2d, tolerance)
+            _new_face = Face3D(new_vertices, self.plane, enforce_right_hand=False)
             return _new_face
-        _new_face._boundary = self._remove_colinear(
+        # the face has holes
+        _boundary = self._remove_colinear(
             self._boundary, self.boundary_polygon2d, tolerance)
-        _new_face._holes = \
-            tuple(self._remove_colinear(hole, self.hole_polygon2d[i], tolerance)
-                  for i, hole in enumerate(self._holes))
+        _holes = tuple(self._remove_colinear(hole, self.hole_polygon2d[i], tolerance)
+                       for i, hole in enumerate(self._holes))
+        _new_face = Face3D(_boundary, self.plane, _holes, enforce_right_hand=False)
         return _new_face
 
     def flip(self):
@@ -818,7 +820,7 @@ class Face3D(Base2DIn3D):
         _plane_int = self._plane.intersect_line_ray(line_ray)
         if _plane_int is not None:
             _int2d = self._plane.xyz_to_xy(_plane_int)
-            if self.polygon2d.is_point_inside_bound_rect(_int2d):
+            if self.polygon2d.is_point_inside_bound_rect(_int2d, Vector2D(1, 0.00001)):
                 return _plane_int
         return None
 
@@ -867,7 +869,7 @@ class Face3D(Base2DIn3D):
         """
         _plane_int = point.project(self._plane.n, self._plane.o)
         _plane_int2d = self._plane.xyz_to_xy(_plane_int)
-        if self.polygon2d.is_point_inside_bound_rect(_plane_int2d):
+        if self.polygon2d.is_point_inside_bound_rect(_plane_int2d, Vector2D(1, 0.00001)):
             return _plane_int
         return None
 
@@ -1736,11 +1738,15 @@ class Face3D(Base2DIn3D):
         triangle formed by 3 vertices is less than the tolerance.
         """
         new_vertices = []
+        skip = 0
         for i, _v in enumerate(pts_2d):
-            _a = pts_2d[i - 2].determinant(pts_2d[i - 1]) + \
-                pts_2d[i - 1].determinant(_v) + _v.determinant(pts_2d[i - 2])
+            _a = pts_2d[i - 2 - skip].determinant(pts_2d[i - 1]) + \
+                pts_2d[i - 1].determinant(_v) + _v.determinant(pts_2d[i - 2 - skip])
             if abs(_a) >= tolerance:
                 new_vertices.append(pts_3d[i - 1])
+                skip = 0
+            else:
+                skip += 1
         return new_vertices
 
     def _is_sub_face(self, face):
@@ -1883,7 +1889,7 @@ class Face3D(Base2DIn3D):
         move_vec = move_vec * (tolerance + 0.00001)
         point_on_face = self.boundary[0] + move_vec
         vert2d = self.plane.xyz_to_xy(point_on_face)
-        if not self.polygon2d.is_point_inside(vert2d):
+        if not self.polygon2d.is_point_inside(vert2d, Vector2D(1, 0.00001)):
             point_on_face = self.boundary[0] - move_vec
         return point_on_face
 
@@ -1911,6 +1917,7 @@ class Face3D(Base2DIn3D):
             normal = [x1 + x2, y1 + y2, z1 + z2]
             # walk around the whole shape to avoid colinear vertices
             for i in range(len(verts) - 2):
+                tri_verts = verts[i:i + 3]
                 x, y, z = Face3D._normal_from_3pts(*verts[i:i + 3])
                 normal[0] += x
                 normal[1] += y
@@ -1934,11 +1941,10 @@ class Face3D(Base2DIn3D):
         # normalize the vectors to make them unit vectors
         d1 = math.sqrt(v1[0] ** 2 + v1[1] ** 2 + v1[2] ** 2)
         d2 = math.sqrt(v2[0] ** 2 + v2[1] ** 2 + v2[2] ** 2)
-        try:
-            v1 = (v1[0] / d1, v1[1] / d1, v1[2] / d1)
-            v2 = (v2[0] / d2, v2[1] / d2, v2[2] / d2)
-        except ZeroDivisionError:  # duplicate vertices in triplet
+        if d1 < 1e-9 or d2 < 1e-9:  # effectively duplicate vertices in triplet
             return (0, 0, 0)
+        v1 = (v1[0] / d1, v1[1] / d1, v1[2] / d1)
+        v2 = (v2[0] / d2, v2[1] / d2, v2[2] / d2)
         # return the cross product
         return (v1[1] * v2[2] - v1[2] * v2[1],
                 -v1[0] * v2[2] + v1[2] * v2[0],
